{---------------------------------------------------------------------------}
{                     COMLIB : Unit de gestion du port serie.               }
{---------------------------------------------------------------------------}
{ Les procdures et fonctions contenues dans ce Unit retournent un numro   }
{ d'erreur dans la fonction ComResult, les codes existants sont les         }
{ suivants :                                                                }
{   0 - L'opration requise c'est droule sans problme.                   }
{   1 - Tentative de lecture d'un port serie vide (aucun caractre dans le  }
{       buffer).                                                            }
{   2 - TIMEOUT. Impossible d'mettre le caractre demand.                 }
{   3 - Le port est ouvert.                                                 }
{   4 - Le port est ferm.                                                  }
{   5 - Erreur dans le numro du port (1 ou 2 autoris).                    }
{   6 - Port non install sur la machine.                                   }
{   7 - Vitesse demand non conforme  la liste disponible (50,75,110,134,  }
{       150,300,600,1200,1800,2000,2400,3600,4800,7200,9600,19200 autoris).}
{   8 - Erreur dans la paritdemand (E,e,P,p,O,o,I,i,N,n,M,m,S,s           }
{       autoriss).                                                         }
{   9 - Longueur de mot demande non conforme  la liste disponible (5,6,7,8}
{       autoris).                                                          }
{  10 - Nombre de bits de stop demands incorrect (1 ou 2 autoriss).       }
{  11 - Erreur dans la taille du buffer (10  65500 autoriss).             }
{                                                                           }
{---------------------------------------------------------------------------}

unit comlib;

interface
uses printer,crt,Dos;

{----------------------------------------------------------------------------}
{ Partie visible. Les procdures et fonctions dclar ici sont "visible"     }
{ par l'utilisateur du unit COMLIB, il peut donc les appeler comme toute     }
{ autre procdure ou fonction du systme.                                    }
{----------------------------------------------------------------------------}

function ComResult : word;
function ComInReady(ComPort : word): Boolean;
function ComInChar(ComPort : word):char;
procedure ClrBuffCom(ComPort : word);
procedure ComBeepOff;
procedure ComBeepOn;
procedure ComOutChar(Comport : word; ch:char);
procedure ComOutStr(NCom : word; BufOut : string);
procedure OpenCom ( ComPort   : word;
                    BaudRate  : word;
                    Parity    : char;
                    WordSize  : word;
                    StopBits  : word;
                    BufferSize: word);
procedure CloseCom(ComPort : word);

{----------------------------------------------------------------------------}
{ Fin de la partie visible. Ici commence la partie "prive".                 }
{----------------------------------------------------------------------------}

implementation

type
  ComBuff = array[1..65500] of char;
var
ComBuffPt         : array[1..2] of ^ComBuff;
CombuffSize       : array[1..2] of word;
SaveExit          : pointer;
SaveHeapError     : pointer;
ComErr            : word;
BeepOn            : boolean;
ComOpen           : array[1..2] of boolean;
ComBuffOverflow : array[1..2] of boolean;
ComBuffEmpty    : array[1..2] of boolean;
BufDeb            : array[1..2] of word;
BufFin            : array[1..2] of word;
TabInt            : array[1..2] of pointer;
ComTable          : array[1..2] of integer absolute $40:0;
{ Table initialise par le bios. Elle contient les adresses }
{ des ports installs. 0 indique rien d'install.           }
{ ComTable retourne la 1re adresse de l'USART.             }

{----------------------------------------------------------------------------}
{      CODE ERREUR DE LA DERNIERE OPERATION. RAZ  chaque appel              }
{----------------------------------------------------------------------------}

function ComResult : word;

begin
  ComResult:=ComErr;
  ComErr:=0;
end;

{----------------------------------------------------------------------------}
{                   INDIQUE SI BUFFER VIDE OU NON                            }
{----------------------------------------------------------------------------}

function ComInReady(ComPort : word):boolean;

begin
  ComErr:=0;
  ComInReady:=false;

  if not (ComPort in [1,2])
    then
      begin
        ComErr:=5;          {  N de port rron }
        exit;
      end;

  if not ComOPen[ComPort]
    then
      begin
        ComErr:=4;          { le port n'est pas ouvert }
        exit;
      end;

  if not ComBuffEmpty[ComPort]
    then ComInReady:=true;
  end;

{---------------------------------------------------------------------------}
{             LECTURE D'UN CARACTERE DANS LE BUFFER                         }
{---------------------------------------------------------------------------}

function ComInChar(ComPort:word):char;

begin
  if ComInReady(ComPort)
    then
      begin
        ComInChar:=ComBuffPt[ComPort]^[BufDeb[ComPort]];
        inc(BufDeb[ComPort]);
        ComBuffOverflow[ComPort]:=false;
        if BufDeb[ComPort]>ComBuffSize[ComPort]
          then BufDeb[ComPort]:=1;
        if BufFIn[ComPort]=BufDeb[ComPort]
          then
            ComBuffEmpty[ComPort]:=true;
        ComErr:=0;
      end
    else
      begin
       ComInChar:=chr(0);
       ComErr:=1;
      end;
end;

{---------------------------------------------------------------------------}
{                        VIDE LE BUFFER                                     }
{---------------------------------------------------------------------------}

procedure ClrBuffCom(ComPort:word);

begin
  ComErr:=0;

  if not (ComPort in [1,2])
    then
      begin
        ComErr:=5;
        exit;
      end;

  if not ComOpen[ComPort]
    then
      begin
        ComErr:=4;
        exit;
      end;

  inline ($FA);                { CLI }
  BufDeb[ComPort]:=1;
  BufFin[ComPort]:=1;
  ComBuffOverflow[ComPort]:=false;
  ComBuffEmpty[ComPort]:=true;
  inline ($FB);
end;

{---------------------------------------------------------------------------}
{                        INHIBE LE BEEP                                     }
{---------------------------------------------------------------------------}

procedure ComBeepOff;

begin
  BeepOn:=false;
end;

{---------------------------------------------------------------------------}
{                       AUTORISE LE BEEP                                    }
{---------------------------------------------------------------------------}

Procedure ComBeepOn;

begin
  BeepOn:=true;
end;

{---------------------------------------------------------------------------}
{                    BEEP FREQUENCE F DUREE D                               }
{---------------------------------------------------------------------------}

procedure beep(F,D:word);

begin
  if BeepOn
    then
      begin
        sound(F);
        delay(D);
        nosound;
      end;
end;

{---------------------------------------------------------------------------}
{$F+} { Les deux procdures d'interruptions sont forc en "FAR CALL"        }
{---------------------------------------------------------------------------}

{---------------------------------------------------------------------------}
{ Procedure appele par interuption  chaque arrive de caractre sur COM1  }
{ place le caractre de com1 dans le buffer.                                }
{---------------------------------------------------------------------------}

function ComInput1(Flags,CS,IP,AX,BX,CX,DX,SI,DI,DS,ES,BP:word):integer;
interrupt;

var
  Car:char;

begin
  Car:=chr(port[ComTable[1]]);
  if not ComBuffOverflow[1]
    then
      begin
        ComBuffPt[1]^[BufFin[1]]:=Car;
        inc(BufFin[1]);
        ComBuffEmpty[1]:=false;
        if BufFin[1]>ComBuffSize[1]
          then BufFin[1]:=1;
        if BufFin[1]=BufDeb[1]
          then
            ComBuffOverflow[1]:=true;
      end
    else
      beep(4000,5);
  Port[$20]:=$20;
end;

{---------------------------------------------------------------------------}
{ Procedure appele par interuption  chaque arrive de caractre sur COM2  }
{ place le caractre de com2 dans le buffer.                                }
{---------------------------------------------------------------------------}

function ComInput2(Flags,CS,IP,AX,BX,CX,DX,SI,DI,DS,ES,BP:word):integer;
interrupt;

var
  Car:char;

begin
  Car:=chr(port[ComTable[2]]);
  if not ComBuffOverflow[2]
    then
      begin
        ComBuffPt[2]^[BufFin[1]]:=Car;
        inc(BufFin[2]);
        ComBuffEmpty[2]:=false;
        if BufFin[2]>ComBuffSize[2]
          then BufFin[2]:=1;
        if BufFin[2]=BufDeb[2]
          then
            ComBuffOverflow[2]:=true;
      end
    else
      beep(4000,5);
  Port[$20]:=$20;
end;

{---------------------------------------------------------------------------}
{$F-}  { Fin du mode "FAR CALL"                                             }
{---------------------------------------------------------------------------}

{---------------------------------------------------------------------------}
{        EMISSION DU CARACTERE Ch SUR LE PORT N ComPort                    }
{---------------------------------------------------------------------------}

procedure ComOutChar(ComPort:word;ch:char);

var
  c:integer;

begin
  if not (ComPort in [1,2])
    then
      begin
        ComErr:=5;
        exit;
      end;
  if not ComOpen[Comport]
    then
      begin
        ComErr:=4;
        exit
      end;
Port[ComTable[ComPort]+4]:=$0B;
c:=maxint;
while (c<>0) and ((Port[ComTable[Comport]+5] and $20)=0) do
  c:=c-1;
if c<>0 then
  begin
    inline ($FA);
    Port[ComTable[ComPort]]:=ord(ch);
    inline ($FB);
    ComErr:=0;
  end
else
  ComErr:=2;
end;

{---------------------------------------------------------------------------}
{      EMISSION DE LA CHAINE BUFOUT SUR LE PORT N COMPORT                  }
{---------------------------------------------------------------------------}

procedure ComOutStr(Ncom:word;BufOut:string);

var
  p:word;

begin
  p:=1;
  repeat
    ComOutChar(NCom,BufOut[p]);
    inc(p);
  until (p>length(BufOut)) or (ComErr<>0);
end;

{---------------------------------------------------------------------------}
{          PROCEDURE D'OUVERTURE D'UN PORT.                                 }
{ ComPort    = N du port 1 ou 2.                                           }
{ BaudRate   = Vitesse de transmission.                                     }
{ Parity     = parit utilis: (P)aire-(I)mpaire-(A)bsence-(M)ark-(S)pace.  }
{ Wordsize   = taille des mots (5,6,7,8 bits).                              }
{ Stopbits   = bits de stop 1 ou 2.                                         }
{ BufferSize = taille du buffer pour le port (10  65500)                   }
{---------------------------------------------------------------------------}

procedure OpenCom (ComPort     :word;
                   BaudRate    :word;
                   Parity      :char;
                   WordSize    :word;
                   StopBits    :word;
                   BufferSize  :word);

var
  Params :word;
  I,M,DV :word;

begin
  ComErr:=0;

  if not (ComPort in [1,2])
    then
      begin
        ComErr:=5;
        exit;
      end;

  if ComOpen[ComPort]
    then
      begin
        ComErr:=3;
        exit;
      end;

  if ComTable[ComPort]=0
    then
      begin
        ComErr:=6;
        exit;
      end;

  case BaudRate of
       50 : DV:=2304;
       75 : DV:=1536;
      110 : DV:=1047;
      134 : DV:=857;
      150 : DV:=768;
      300 : DV:=384;
      600 : DV:=192;
     1200 : DV:=96;
     1800 : DV:=64;
     2000 : DV:=58;
     2400 : DV:=48;
     3600 : DV:=32;
     4800 : DV:=24;
     7200 : DV:=16;
     9600 : DV:=12;
    19200 : DV:=6;
  else
    begin
      ComErr:=7;
      exit;
    end;
  end;

Params:=0;
case Parity of
  'E','e','P','p' : Params:=Params or $0018; {Paire}
  'O','o','I','i' : Params:=Params or $0008; {Impaire}
  'N','n','A','a' : Params:=Params or $0000; {Pas de parit}
  'M','m'         : Params:=Params or $0028; {Parit "Mark"}
  'S','s'         : Params:=Params or $0038; {Parit "Space"}
else
  begin
    ComErr:=8;
    exit;
  end;
end;

case StopBits of
  1 : Params:=Params or $0000;
  2 : Params:=Params or $0004;
else
  begin
    ComErr:=10;
    exit;
  end;
end;

case WordSize of
  5 : Params:=Params or $0000;
  6 : Params:=Params or $0001;
  7 : Params:=Params or $0002;
  8 : Params:=Params or $0003;
else
  begin
    ComErr:=9;
    exit;
  end;
end;

Params:=Params and $7F;

if (BufferSize<10) or (BufferSize >65500)
  then
    begin
      ComErr:=11;
      exit;
    end
  else
    begin
      getmem(ComBuffPt[ComPort],BufferSize);
      ComBuffSize[ComPort]:=BufferSize;
      BufDeb[Comport]:=1;
      BufFin[ComPort]:=1;
      ComBuffOverflow[ComPort]:=false;
      ComBuffEmpty[ComPort]:=true;
      ComOpen[ComPort]:=true;
    end;

case ComPort of
  1 : SetIntVec(12,@ComInput1);
  2 : SetIntVec(11,@ComInput2);
end;

inline ($FA);      {CLI}

Port[$20]:=$20;

Port[ComTable[ComPort]+3]:=Port[ComTable[ComPort]+3] or $80;
Port[ComTable[ComPort]]:=LO(DV);
Port[ComTable[ComPort]+1]:=HI(DV);
Port[ComTable[ComPort]+3]:=Params;

I:=Port[ComTable[ComPort]+5];
I:=Port[ComTable[ComPort]];
i:=Port[$21];
if ComPort=1
  then M:=$EF
  else M:=$F7;
Port[$21]:=I and M;

Port[ComTable[ComPort]+1]:=$01;
I:=Port[ComTable[ComPort]+4];
Port[ComTable[ComPort]+4]:=I or $0B;

inline ($FB);
end;

{---------------------------------------------------------------------------}
{                 FERMETURE DU PORT N COMPORT                              }
{---------------------------------------------------------------------------}

procedure CloseCom(ComPort:word);

var
  I,M :integer;

begin
  ComErr:=0;
  if not (ComPort in [1,2])
    then
      begin
        ComErr:=5;
        exit;
      end;

  if not ComOpen[ComPort]
    then
      begin
        ComErr:=4;
        exit;
      end;

  if not ComTable[ComPort]=0
    then
      begin
        ComErr:=6;
        exit;
      end;

  if LongInt(ComBuffPt[ComPort])<LongInt(HeapPtr)
    then FreeMem(ComBuffpt[ComPort],ComBuffSize[ComPort]);
         ComBuffPt[ComPort]:=nil;
         ComOpen[Comport]:=false;
         SetIntVec(13-ComPort,TabInt[ComPort]);
         inline ($FA);  {CLI}

  I:=Port[$21];
  if ComPort=1
    then m:=$10
    else m:=$08;
  Port[$21]:=I or M;

 Port[ComTable[ComPort]+1]:=0;
 Port[ComTable[ComPort]+4]:=0;

 inline ($FB);
end;

{---------------------------------------------------------------------------}
{        PROCEDURE DE SORTIE. MODE "FAR CALL"                               }
{---------------------------------------------------------------------------}

{$F+} procedure ComExit; {$F-}
begin
  if ComOpen[1]
    then CloseCom(1);
  if ComOpen[2]
    then CloseCom(2);
  ExitProc:=SaveExit;
  HeapError:=SaveHeapError;
end;

{---------------------------------------------------------------------------}
{                    INITIALISATIONS                                        }
{---------------------------------------------------------------------------}

begin
  SaveExit:=ExitProc;
  ExitProc:=@ComExit;
  GetIntVec(12,TabInt[1]);
  GetIntVec(11,TabInt[2]);
  ComBuffPt[1]:=nil;
  ComBuffPt[2]:=nil;
  ComOpen[1]:=False;
  ComOpen[2]:=False;
  ComErr:=0;
  BeepOn:=true;
 end.
 {---------------------------------------------------------------------------}
